* Output options: all can be turned on/off by adding or removing the 'NO':
  page number, date, centering, or page breaks, page length and width;
OPTIONS Spool NOnumber NOdate NOcenter FormDlim=' ' PageSize=MAX LineSize=140;
* Eliminate SAS default titles and names of tables in output (TRACE ON to show);
TITLE; ODS TRACE OFF; ODS GRAPHICS ON; ODS HTML CLOSE; ODS HTML;

* Location for files to be saved - CHANGE PATH;
%LET example= F:\13_Psyc930Generalized\930_Download_Part2; 
LIBNAME example "&example.";

* Import data;
PROC IMPORT DATAFILE="&example.\lesaMA.sav" OUT=work.MA DBMS=sav REPLACE; RUN;

* Recode variables;
DATA work.MA; SET work.MA; 
	* Remove formats;
	FORMAT _ALL_;
	* Remove cases with missing data on predictors;
	IF NMISS(age,gender)>0 THEN DELETE;
	* Rename gender to be interpretable;
	female = gender;
	* Create piecewise age slopes;
	     IF agegroup=0 THEN over75=0;
	ELSE IF agegroup=1 THEN over75=age-75;
	LABEL female= "0=M, 1=F" 
          over75= "Years over 75";
	* Get mean RT across trials;
	MeanRT = MEAN(OF rtsec1--rtsec80);
	LogMeanRT = LOG(MeanRT);
	* Get log of per-condition RT;
	LogRT11=LOG(RT11); LogRT12=LOG(RT12); LogRT21=LOG(RT21); LogRT22=LOG(RT22);
RUN;

* Distribution of mean RT;
PROC UNIVARIATE NOPRINT DATA=work.MA; 
	VAR meanRT; HISTOGRAM meanRT / NORMAL(MU=EST SIGMA=EST) 
							       LOGNORMAL(SCALE=EST SHAPE=EST)
                                   GAMMA(SCALE=EST SHAPE=EST); 
RUN;

***** LOGNORMAL UNIVARIATE MODELS;

* Save SAS syntax and output to permanent text file;
PROC PRINTTO PRINT="&example.\2_LogNormal_Univariate_Models.txt" 
			 LOG=  "&example.\2_LogNormal_Univariate_Models.txt"; 
RUN;

TITLE1 "EMPTY MODEL PREDICTING LOG-TRANSFORMED RT, LINK=IDENTITY, DIST=NORMAL";
PROC GLIMMIX DATA=work.MA NOCLPRINT NOITPRINT GRADIENT METHOD=QUAD; 
  MODEL LogMeanRT = / SOLUTION LINK=IDENTITY DIST=NORMAL;
* Requesting logit intercept to be transformed into probability;
  ESTIMATE "Intercept" intercept 1 / ILINK;
RUN; TITLE1; 

TITLE1 "EMPTY MODEL PREDICTING ORIGINAL RT, LINK=IDENTITY, DIST=LOGNORMAL";
PROC GLIMMIX DATA=work.MA NOCLPRINT NOITPRINT GRADIENT METHOD=QUAD; 
  MODEL MeanRT = / SOLUTION LINK=IDENTITY DIST=LOGNORMAL;
* Requesting logit intercept to be transformed into probability;
  ESTIMATE "Intercept" intercept 1 / ILINK;
RUN; TITLE1; 

TITLE1 "AGE PIECEWISE LINEAR MODEL PREDICTING ORIGINAL RT, LINK=IDENTITY, DIST=LOGNORMAL";
PROC GLIMMIX DATA=work.MA NOCLPRINT NOITPRINT GRADIENT METHOD=QUAD; 
  MODEL MeanRT = agegroup over75 / SOLUTION LINK=IDENTITY DIST=LOGNORMAL;
* Requesting log intercept to be transformed into amount;
  ESTIMATE "Intercept for YA"       intercept 1 agegroup 0 over75  0 / ILINK;
  ESTIMATE "Intercept for OA Age70" intercept 1 agegroup 1 over75 -5 / ILINK;
  ESTIMATE "Intercept for OA Age75" intercept 1 agegroup 1 over75  0 / ILINK;
  ESTIMATE "Intercept for OA Age80" intercept 1 agegroup 1 over75  5 / ILINK;
  CONTRAST "DF=2 Total Age Effect"  agegroup 1, over75 1 / CHISQ;
  OUTPUT OUT=Age PRED(ILINK)=PredMeanRT;
RUN; TITLE1; 

TITLE1 "Prediction by Piecewise Linear Effect of Age";
PROC GPLOT DATA=Age; PLOT LogMeanRT*PredMeanRT; RUN; QUIT; 
PROC CORR DATA=Age; VAR LogMeanRT PredMeanRT; RUN; TITLE1;

TITLE1 "AGE PIECEWISE QUADRATIC MODEL PREDICTING ORIGINAL RT, LINK=IDENTITY, DIST=LOGNORMAL";
PROC GLIMMIX DATA=work.MA NOCLPRINT NOITPRINT GRADIENT METHOD=QUAD; 
  MODEL MeanRT = agegroup over75 over75*over75 / SOLUTION LINK=IDENTITY DIST=LOGNORMAL;
* Requesting log intercept to be transformed into amount;
  ESTIMATE "Intercept for YA"       intercept 1 agegroup 0 over75  0 over75*over75   0 / ILINK CL;
  ESTIMATE "Intercept for OA Age70" intercept 1 agegroup 1 over75 -5 over75*over75 -25 / ILINK CL;
  ESTIMATE "Intercept for OA Age75" intercept 1 agegroup 1 over75  0 over75*over75   0 / ILINK CL;
  ESTIMATE "Intercept for OA Age80" intercept 1 agegroup 1 over75  5 over75*over75  25 / ILINK CL;
  CONTRAST "DF=2 Over 75 Age Effect"  over75 1, over75*over75 1 / CHISQ;
  ESTIMATE "Linear Age Slope at 70"	over75 1 over75*over75 -10; * twice quad is how linear changes;
  ESTIMATE "Linear Age Slope at 75"	over75 1 over75*over75   0;
  ESTIMATE "Linear Age Slope at 80"	over75 1 over75*over75  10;
  OUTPUT OUT=Age2 PRED(ILINK)=PredMeanRT;
RUN; TITLE1; 

TITLE1 "Prediction by Piecewise Quadratic Effect of Age";
PROC GPLOT DATA=Age2; PLOT LogMeanRT*PredMeanRT; RUN; QUIT; 
PROC CORR DATA=Age2; VAR LogMeanRT PredMeanRT; RUN; TITLE1;


***** GAMMA UNIVARIATE MODELS;

* Save SAS syntax and output to permanent text file;
PROC PRINTTO PRINT="&example.\2_Gamma_Univariate_Models.txt" 
			 LOG=  "&example.\2_Gamma_Univariate_Models.txt"; 
RUN;

TITLE1 "EMPTY MODEL PREDICTING ORIGINAL RT, LINK=LOG, DIST=GAMMA";
PROC GLIMMIX DATA=work.MA NOCLPRINT NOITPRINT GRADIENT METHOD=QUAD; 
  MODEL MeanRT = / SOLUTION LINK=LOG DIST=GAMMA;
* Requesting logit intercept to be transformed into probability;
  ESTIMATE "Intercept" intercept 1 / ILINK;
RUN; TITLE1; 

TITLE1 "AGE PIECEWISE LINEAR MODEL PREDICTING ORIGINAL RT, LINK=LOG, DIST=GAMMA";
PROC GLIMMIX DATA=work.MA NOCLPRINT NOITPRINT GRADIENT METHOD=QUAD; 
  MODEL MeanRT = agegroup over75 / SOLUTION LINK=LOG DIST=GAMMA;
* Requesting log intercept to be transformed into amount;
  ESTIMATE "Intercept for YA"       intercept 1 agegroup 0 over75  0 / ILINK;
  ESTIMATE "Intercept for OA Age70" intercept 1 agegroup 1 over75 -5 / ILINK;
  ESTIMATE "Intercept for OA Age75" intercept 1 agegroup 1 over75  0 / ILINK;
  ESTIMATE "Intercept for OA Age80" intercept 1 agegroup 1 over75  5 / ILINK;
  CONTRAST "DF=2 Total Age Effect"  agegroup 1, over75 1 / CHISQ;
  OUTPUT OUT=Age PRED(ILINK)=PredMeanRT;
RUN; TITLE1; 

TITLE1 "Prediction by Piecewise Linear Effect of Age";
PROC GPLOT DATA=Age; PLOT LogMeanRT*PredMeanRT; RUN; QUIT; 
PROC CORR DATA=Age; VAR LogMeanRT PredMeanRT; RUN; TITLE1;

TITLE1 "AGE PIECEWISE QUADRATIC MODEL PREDICTING ORIGINAL RT, LINK=LOG, DIST=GAMMA";
PROC GLIMMIX DATA=work.MA NOCLPRINT NOITPRINT GRADIENT METHOD=QUAD; 
  MODEL MeanRT = agegroup over75 over75*over75 / SOLUTION LINK=LOG DIST=GAMMA;
* Requesting log intercept to be transformed into amount;
  ESTIMATE "Intercept for YA"       intercept 1 agegroup 0 over75  0 over75*over75   0 / ILINK CL;
  ESTIMATE "Intercept for OA Age70" intercept 1 agegroup 1 over75 -5 over75*over75 -25 / ILINK CL;
  ESTIMATE "Intercept for OA Age75" intercept 1 agegroup 1 over75  0 over75*over75   0 / ILINK CL;
  ESTIMATE "Intercept for OA Age80" intercept 1 agegroup 1 over75  5 over75*over75  25 / ILINK CL;
  CONTRAST "DF=2 Over 75 Age Effect"  over75 1, over75*over75 1 / CHISQ;
  ESTIMATE "Linear Age Slope at 70"	over75 1 over75*over75 -10; * twice quad is how linear changes;
  ESTIMATE "Linear Age Slope at 75"	over75 1 over75*over75   0;
  ESTIMATE "Linear Age Slope at 80"	over75 1 over75*over75  10;
  OUTPUT OUT=Age2 PRED(ILINK)=PredMeanRT;
RUN; TITLE1; 

TITLE1 "Prediction by Piecewise Quadratic Effect of Age";
PROC GPLOT DATA=Age2; PLOT LogMeanRT*PredMeanRT; RUN; QUIT; 
PROC CORR DATA=Age2; VAR LogMeanRT PredMeanRT; RUN; TITLE1;

* Close output;
PROC PRINTTO; RUN; ODS HTML CLOSE; ODS HTML;


***** SET UP FOR MULTIVARIATE MODELS;

* Create format to use for condition variables;
PROC FORMAT; 
	VALUE Fcondition 0="0Low" 1="1High";
	VALUE Fagegroup 0="0Younger" 1="1Older";
RUN;

* Stack data into multivariate;
DATA work.MAstack; SET work.MA;
	DistN=1; DistL=11; DistG=5; LinkN=1; LinkL=1; LinkG=4; * Add distribution and links for normal, lognormal, gamma;
	DV="00-LM-LS"; StackRT=RT11; StackLogRT=LogRT11; Mean=0; Sal=0; OUTPUT; * Low  meaning, low  salience;
	DV="01-LM-HS"; StackRT=RT12; StackLogRT=LogRT12; Mean=0; Sal=1; OUTPUT; * Low  meaning, high salience;
	DV="10-HM-LS"; StackRT=RT21; StackLogRT=LogRT21; Mean=1; Sal=0; OUTPUT; * High meaning, low  salience;
	DV="11-HM-HS"; StackRT=RT22; StackLogRT=LogRT22; Mean=1; Sal=1; OUTPUT; * High meaning, high salience;
RUN;
PROC SORT DATA=work.MAstack; BY DV subnum; RUN;

* Distribution of mean RT by agegroup;
PROC UNIVARIATE NOPRINT DATA=work.MAstack; CLASS agegroup DV;
	VAR StackRT; HISTOGRAM StackRT / NORMAL(MU=EST SIGMA=EST) 
							         LOGNORMAL(SCALE=EST SHAPE=EST)
                                     GAMMA(SCALE=EST SHAPE=EST); 
RUN;

TITLE1 "MULTIVARIATE MODEL PREDICTING LOG RT -- CHECKING PREDICTED COVARIANCE MATRIX";
PROC MIXED DATA=work.MAstack NOCLPRINT NOITPRINT COVTEST METHOD=ML NOBOUND; 
	CLASS subnum DV mean sal agegroup; 
	FORMAT mean Fcondition. sal Fcondition. agegroup Fagegroup.;
	MODEL StackLogRT  = mean|sal|agegroup over75 / ;
	REPEATED DV / TYPE=UN SUBJECT=subnum R=1,98 GROUP=agegroup;
RUN; TITLE1;

* Create dummy codes to use in creating heterogeneous variances and covariances;
DATA work.MAstack; SET work.MAstack;
	     IF Mean=0 AND Sal=0 THEN DO; c00=1; c01=0; c10=0; c11=0; cM=0; END; 
	ELSE IF Mean=0 AND Sal=1 THEN DO; c00=0; c01=1; c10=0; c11=0; cM=1; END; 
	ELSE IF Mean=1 AND Sal=0 THEN DO; c00=0; c01=0; c10=1; c11=0; cM=1; END;
	ELSE IF Mean=1 AND Sal=1 THEN DO; c00=0; c01=0; c10=0; c11=1; cM=0; END; 
RUN;


***** LOGNORMAL MULTIVARIATE MODELS;

* Save SAS syntax and output to permanent text file;
PROC PRINTTO PRINT="&example.\2_LogNormal_Multivariate_Models.txt" 
			 LOG=  "&example.\2_LogNormal_Multivariate_Models.txt"; 
RUN;

TITLE1 "MULTIVARIATE MODEL PREDICTING LOG RT -- CHECKING PREDICTED COVARIANCE MATRIX";
TITLE2 "RANDOM INTERCEPT ONLY";
PROC MIXED DATA=work.MAstack NOCLPRINT NOITPRINT COVTEST METHOD=ML NOBOUND; 
	CLASS subnum mean sal agegroup; 
	FORMAT mean Fcondition. sal Fcondition. agegroup Fagegroup.;
	MODEL StackLogRT  = mean|sal|agegroup over75 / ;
	RANDOM INTERCEPT / TYPE=VC SUBJECT=subnum V=1,98 GROUP=agegroup;
RUN; TITLE1; TITLE2;

TITLE1 "MULTIVARIATE MODEL PREDICTING ORIGINAL RT, LINK=IDENTITY, DIST=LOGNORMAL";
TITLE2 "RANDOM INTERCEPT ONLY";
PROC GLIMMIX DATA=work.MAstack NOCLPRINT NOITPRINT GRADIENT METHOD=LAPLACE NOBOUND; 
	CLASS subnum mean sal agegroup; 
	FORMAT mean Fcondition. sal Fcondition. agegroup Fagegroup.;
	MODEL StackRT  = mean|sal|agegroup over75 / LINK=BYOBS(LinkL) DIST=BYOBS(DistL);
	RANDOM INTERCEPT / TYPE=VC SUBJECT=subnum GROUP=agegroup;
	COVTEST "Homogeneous Variances by Age?" HOMOGENEITY;
RUN; TITLE1; TITLE2;

TITLE1 "MULTIVARIATE MODEL PREDICTING LOG RT -- CHECKING PREDICTED COVARIANCE MATRIX";
TITLE2 "RANDOM INTERCEPT ONLY, +c00";
PROC MIXED DATA=work.MAstack NOCLPRINT NOITPRINT COVTEST METHOD=ML NOBOUND; 
	CLASS subnum mean sal agegroup; 
	FORMAT mean Fcondition. sal Fcondition. agegroup Fagegroup.;
	MODEL StackLogRT  = mean|sal|agegroup over75 / ;
	RANDOM INTERCEPT c00 / TYPE=VC SUBJECT=subnum V=1,98 GROUP=agegroup;
RUN; TITLE1; TITLE2;

TITLE1 "MULTIVARIATE MODEL PREDICTING ORIGINAL RT, LINK=IDENTITY, DIST=LOGNORMAL";
TITLE2 "RANDOM INTERCEPT ONLY, +c00";
PROC GLIMMIX DATA=work.MAstack NOCLPRINT NOITPRINT GRADIENT METHOD=LAPLACE NOBOUND; 
	CLASS subnum mean sal agegroup; 
	FORMAT mean Fcondition. sal Fcondition. agegroup Fagegroup.;
	MODEL StackRT  = mean|sal|agegroup over75 / LINK=BYOBS(LinkL) DIST=BYOBS(DistL);
	RANDOM INTERCEPT c00 / TYPE=VC SUBJECT=subnum GROUP=agegroup;
	COVTEST "Homogeneous Variances by Age?" HOMOGENEITY;
	COVTEST "YA group need c00?"     . 0 . . .;
	COVTEST "OA group need c00?"     . . . 0 .;
	COVTEST "Either group need c00?" . 0 . 0 .;
RUN; TITLE1; TITLE2;

TITLE1 "MULTIVARIATE MODEL PREDICTING LOG RT -- CHECKING PREDICTED COVARIANCE MATRIX";
TITLE2 "RANDOM INTERCEPT ONLY +c00 +c11";
PROC MIXED DATA=work.MAstack NOCLPRINT NOITPRINT COVTEST METHOD=ML NOBOUND; 
	CLASS subnum mean sal agegroup; 
	FORMAT mean Fcondition. sal Fcondition. agegroup Fagegroup.;
	MODEL StackLogRT  = mean|sal|agegroup over75 / ;
	RANDOM INTERCEPT c00 c11 / TYPE=VC SUBJECT=subnum V=1,98 GROUP=agegroup;
RUN; TITLE1; TITLE2;

TITLE1 "MULTIVARIATE MODEL PREDICTING ORIGINAL RT, LINK=IDENTITY, DIST=LOGNORMAL";
TITLE2 "RANDOM INTERCEPT ONLY +c00 +c11";
PROC GLIMMIX DATA=work.MAstack NOCLPRINT NOITPRINT GRADIENT METHOD=LAPLACE NOBOUND; 
	CLASS subnum mean sal agegroup; 
	FORMAT mean Fcondition. sal Fcondition. agegroup Fagegroup.;
	MODEL StackRT  = mean|sal|agegroup over75 / LINK=BYOBS(LinkL) DIST=BYOBS(DistL);
	RANDOM INTERCEPT c00 c11 / TYPE=VC SUBJECT=subnum GROUP=agegroup;
	COVTEST "Homogeneous Variances by Age?" HOMOGENEITY;
	COVTEST "YA group need c11?"     . . 0 . . . .;
	COVTEST "OA group need c11?"     . . . . . 0 .;
	COVTEST "Either group need c11?" . . 0 . . 0 .;
RUN; TITLE1; TITLE2;

* Close output;
PROC PRINTTO; RUN; ODS HTML CLOSE; ODS HTML;


***** GAMMA MULTIVARIATE MODELS;

* Save SAS syntax and output to permanent text file;
PROC PRINTTO PRINT="&example.\2_Gamma_Multivariate_Models.txt" 
			 LOG=  "&example.\2_Gamma_Multivariate_Models.txt"; 
RUN;

TITLE1 "MULTIVARIATE MODEL PREDICTING ORIGINAL RT, LINK=LOG, DIST=GAMMA";
TITLE2 "RANDOM INTERCEPT ONLY";
PROC GLIMMIX DATA=work.MAstack NOCLPRINT NOITPRINT GRADIENT METHOD=LAPLACE NOBOUND; 
	CLASS subnum mean sal agegroup; 
	FORMAT mean Fcondition. sal Fcondition. agegroup Fagegroup.;
	MODEL StackRT  = mean|sal|agegroup over75 / LINK=BYOBS(LinkG) DIST=BYOBS(DistG);
	RANDOM INTERCEPT / TYPE=VC SUBJECT=subnum GROUP=agegroup;
	COVTEST "Homogeneous Variances by Age?" HOMOGENEITY;
RUN; TITLE1; TITLE2;

TITLE1 "MULTIVARIATE MODEL PREDICTING ORIGINAL RT, LINK=LOG, DIST=GAMMA";
TITLE2 "RANDOM INTERCEPT ONLY, +c00";
PROC GLIMMIX DATA=work.MAstack NOCLPRINT NOITPRINT GRADIENT METHOD=LAPLACE NOBOUND; 
	CLASS subnum mean sal agegroup; 
	FORMAT mean Fcondition. sal Fcondition. agegroup Fagegroup.;
	MODEL StackRT  = mean|sal|agegroup over75 / LINK=BYOBS(LinkG) DIST=BYOBS(DistG);
	RANDOM INTERCEPT c00 / TYPE=VC SUBJECT=subnum GROUP=agegroup;
	COVTEST "Homogeneous Variances by Age?" HOMOGENEITY;
RUN; TITLE1; TITLE2;

TITLE1 "MULTIVARIATE MODEL PREDICTING ORIGINAL RT, LINK=LOG, DIST=GAMMA";
TITLE2 "RANDOM INTERCEPT ONLY +c00 +c11";
PROC GLIMMIX DATA=work.MAstack NOCLPRINT NOITPRINT GRADIENT METHOD=LAPLACE NOBOUND; 
	CLASS subnum mean sal agegroup; 
	FORMAT mean Fcondition. sal Fcondition. agegroup Fagegroup.;
	MODEL StackRT  = mean|sal|agegroup over75 / LINK=BYOBS(LinkG) DIST=BYOBS(DistG);
	RANDOM INTERCEPT c00 c11 / TYPE=VC SUBJECT=subnum GROUP=agegroup;
	COVTEST "Homogeneous Variances by Age?" HOMOGENEITY;
RUN; TITLE1; TITLE2;

* Close output;
PROC PRINTTO; RUN; ODS HTML CLOSE; ODS HTML;



