* Output options: all can be turned on/off by adding or removing the 'NO':
  page number, date, centering, or page breaks, page length and width;
OPTIONS Spool NOnumber NOdate NOcenter FormDlim=' ' PageSize=MAX LineSize=140;
* Eliminate SAS default titles and names of tables in output (TRACE ON to show);
TITLE; ODS TRACE OFF; ODS GRAPHICS ON; ODS HTML CLOSE; ODS HTML;

* Location for files to be saved - CHANGE PATH;
%LET example= F:\13_Psyc930Generalized\930_Download_Part2; 
LIBNAME example "&example.";

* Import data;
PROC IMPORT DATAFILE="&example.\lesaMA.sav" OUT=work.MA DBMS=sav REPLACE; RUN;

* Recode variables;
DATA work.MA; SET work.MA; 
	* Remove formats;
	FORMAT _ALL_;
	* Remove cases with missing data on predictors;
	IF NMISS(age,gender)>0 THEN DELETE;
	* Rename gender to be interpretable;
	female = gender;
	* Create piecewise age slopes;
	     IF agegroup=0 THEN over75=0;
	ELSE IF agegroup=1 THEN over75=age-75;
	LABEL female= "0=M, 1=F" 
          over75= "Years over 75";
	* Get number of errors;
	Nerror = SUM(OF error1--error80);
RUN;

* Full distribution of number of errors;
PROC UNIVARIATE DATA=work.MA; CLASS agegroup; VAR Nerror; HISTOGRAM Nerror; 
RUN; 

* Export to Mplus to verify models;
DATA work.ForMplus; SET work.MA; KEEP Subnum agegroup over75 Nerror; RUN;
PROC EXPORT DATA=work.ForMplus OUTFILE="&example.\MA_Mplus.csv" DBMS=CSV REPLACE; 
PUTNAMES=NO; RUN;

***** POISSON AND NEGATIVE BINOMIAL UNIVARIATE MODELS FOR ERRORS;

* Save SAS syntax and output to permanent text file;
PROC PRINTTO PRINT="&example.\2_Count_Univariate_Models.txt" 
			 LOG=  "&example.\2_Count_Univariate_Models.txt"; 
RUN;

TITLE1 "EMPTY MODEL PREDICTING ERRORS DV, LOG LINK, POISSON DIST";
PROC GLIMMIX DATA=work.MA NOCLPRINT GRADIENT METHOD=QUAD; 
  MODEL Nerror = / SOLUTION LINK=LOG DIST=POISSON;
* Requesting log intercept to be transformed into count;
  ESTIMATE "Intercept" intercept 1 / ILINK;
RUN; TITLE1; 

TITLE1 "EMPTY MODEL PREDICTING ERRORS DV, LOG LINK, NEGATIVE BINOMIAL DIST";
PROC GLIMMIX DATA=work.MA NOCLPRINT GRADIENT METHOD=QUAD; 
  MODEL Nerror = / SOLUTION LINK=LOG DIST=NB;
* Requesting log intercept to be transformed into count;
  ESTIMATE "Intercept" intercept 1 / ILINK;
RUN; TITLE1; 

TITLE1 "AGE PIECEWISE LINEAR MODEL PREDICTING ERRORS DV, LOG LINK, NEGATIVE BINOMIAL DIST";
PROC GLIMMIX DATA=work.MA NOCLPRINT GRADIENT METHOD=QUAD; 
  MODEL Nerror = agegroup over75 / SOLUTION LINK=LOG DIST=NB;
* Requesting logit intercept to be transformed into probability;
  ESTIMATE "Intercept for YA"       intercept 1 agegroup 0 over75  0 / ILINK;
  ESTIMATE "Intercept for OA Age70" intercept 1 agegroup 1 over75 -5 / ILINK;
  ESTIMATE "Intercept for OA Age75" intercept 1 agegroup 1 over75  0 / ILINK;
  ESTIMATE "Intercept for OA Age80" intercept 1 agegroup 1 over75  5 / ILINK;
  CONTRAST "DF=2 Total Age Effect"  agegroup 1, over75 1 / CHISQ;
  OUTPUT OUT=Age PRED(ILINK)=PredError;
RUN; TITLE1; 

TITLE1 "Prediction by Piecewise Linear Effect of Age";
PROC GPLOT DATA=Age; PLOT Nerror*PredError; RUN; QUIT; 
PROC CORR DATA=Age; VAR Nerror PredError; RUN; TITLE1;

TITLE1 "AGE PIECEWISE QUADRATIC MODEL PREDICTING ERRORS DV, LOG LINK, NEGATIVE BINOMIAL DIST";
PROC GLIMMIX DATA=work.MA NOCLPRINT GRADIENT METHOD=QUAD; 
  MODEL Nerror = agegroup over75 over75*over75 / SOLUTION LINK=LOG DIST=NB;
* Requesting logit intercept to be transformed into probability;
  ESTIMATE "Intercept for YA"       intercept 1 agegroup 0 over75  0 over75*over75   0 / ILINK CL;
  ESTIMATE "Intercept for OA Age70" intercept 1 agegroup 1 over75 -5 over75*over75 -25 / ILINK CL;
  ESTIMATE "Intercept for OA Age75" intercept 1 agegroup 1 over75  0 over75*over75   0 / ILINK CL;
  ESTIMATE "Intercept for OA Age80" intercept 1 agegroup 1 over75  5 over75*over75  25 / ILINK CL;
  CONTRAST "DF=2 Over 75 Age Effect"  over75 1, over75*over75 1 / CHISQ;
  ESTIMATE "Linear Age Slope at 70"	over75 1 over75*over75 -10; * twice quad is how linear changes;
  ESTIMATE "Linear Age Slope at 75"	over75 1 over75*over75   0;
  ESTIMATE "Linear Age Slope at 80"	over75 1 over75*over75  10;
  OUTPUT OUT=Age2 PRED(ILINK)=PredError;
RUN; TITLE1; 

TITLE1 "Prediction by Piecewise Quadratic Effect of Age";
PROC GPLOT DATA=Age2; PLOT Nerror*PredError; RUN; QUIT; 
PROC CORR DATA=Age2; VAR Nerror PredError; RUN; TITLE1;

TITLE1 "AGE PIECEWISE QUADRATIC MODEL PREDICTING ERRORS DV, LOG LINK, POISSON DIST";
TITLE2 "SCALE FACTOR STILL NEEDED?";
PROC GLIMMIX DATA=work.MA NOCLPRINT GRADIENT METHOD=QUAD; 
  MODEL Nerror = agegroup over75 over75*over75 / SOLUTION LINK=LOG DIST=POISSON;
* Requesting logit intercept to be transformed into probability;
  ESTIMATE "Intercept for YA"       intercept 1 agegroup 0 over75  0 over75*over75   0 / ILINK CL;
  ESTIMATE "Intercept for OA Age70" intercept 1 agegroup 1 over75 -5 over75*over75 -25 / ILINK CL;
  ESTIMATE "Intercept for OA Age75" intercept 1 agegroup 1 over75  0 over75*over75   0 / ILINK CL;
  ESTIMATE "Intercept for OA Age80" intercept 1 agegroup 1 over75  5 over75*over75  25 / ILINK CL;
  CONTRAST "DF=2 Over 75 Age Effect"  over75 1, over75*over75 1 / CHISQ;
  ESTIMATE "Linear Age Slope at 70"	over75 1 over75*over75 -10; * twice quad is how linear changes;
  ESTIMATE "Linear Age Slope at 75"	over75 1 over75*over75   0;
  ESTIMATE "Linear Age Slope at 80"	over75 1 over75*over75  10;
  OUTPUT OUT=Age2 PRED(ILINK)=PredError;
RUN; TITLE1; TITLE2; 

* Close output;
PROC PRINTTO; RUN; ODS HTML CLOSE; ODS HTML;



***** ZERO-INFLATED POISSON AND NEGATIVE BINOMIAL UNIVARIATE MODELS FOR ERRORS;

* Save SAS syntax and output to permanent text file;
PROC PRINTTO PRINT="&example.\2_Inflated_Univariate_Models.txt" 
			 LOG=  "&example.\2_Inflated_Univariate_Models.txt"; 
RUN;

TITLE1 "EMPTY MODEL PREDICTING ERRORS DV, LOG LINK, POISSON DIST";
PROC GENMOD DATA=work.MA; 
  MODEL Nerror = / LINK=LOG DIST=POISSON;
* Requesting log intercept to be transformed into count;
  ESTIMATE "Intercept for Count"           intercept 1;
RUN; TITLE1; 

TITLE1 "EMPTY MODEL PREDICTING ERRORS DV, LOG LINK, NEGATIVE BINOMIAL DIST";
PROC GENMOD DATA=work.MA; 
  MODEL Nerror = / LINK=LOG DIST=NB;
* Requesting log intercept to be transformed into count;
  ESTIMATE "Intercept for Count"           intercept 1;
RUN; TITLE1; 

TITLE1 "EMPTY MODEL PREDICTING ERRORS DV, LOG LINK, ZERO-INFLATED POISSON DIST";
PROC GENMOD DATA=work.MA; 
  MODEL Nerror = / LINK=LOG DIST=ZIP;
  ZEROMODEL / LINK=LOGIT; 
* Requesting log intercept to be transformed into count;
  ESTIMATE "Intercept for Inflation" @ZERO intercept 1;
  ESTIMATE "Intercept for Count"           intercept 1;
RUN; TITLE1; 

TITLE1 "EMPTY MODEL PREDICTING ERRORS DV, LOG LINK, ZERO-INFLATED NB DIST";
PROC GENMOD DATA=work.MA; 
  MODEL Nerror = / LINK=LOG DIST=ZINB;
  ZEROMODEL / LINK=LOGIT; 
* Requesting log intercept to be transformed into count;
  ESTIMATE "Intercept for Inflation" @ZERO intercept 1;
  ESTIMATE "Intercept for Count"           intercept 1;
RUN; TITLE1; 

TITLE1 "AGE PIECEWISE LINEAR MODEL PREDICTING ERRORS DV, LOG LINK, ZERO-INFLATED NB DIST";
PROC GENMOD DATA=work.MA; 
  MODEL Nerror = agegroup over75 / LINK=LOG DIST=ZINB;
  ZEROMODEL  / LINK=LOGIT; 
* Requesting log intercept to be transformed into count;
  ESTIMATE "Count Intercept for YA"       intercept 1 agegroup 0 over75  0;
  ESTIMATE "Count Intercept for OA Age70" intercept 1 agegroup 1 over75 -5;
  ESTIMATE "Count Intercept for OA Age75" intercept 1 agegroup 1 over75  0;
  ESTIMATE "Count Intercept for OA Age80" intercept 1 agegroup 1 over75  5;
  CONTRAST "DF=2 Total Age Effect"  agegroup 1, over75 1;
RUN; TITLE1; 

TITLE1 "AGE PIECEWISE LINEAR MODEL PREDICTING ERRORS DV, LOG LINK, ZERO-INFLATED NB DIST";
TITLE2 "ADDING AGE GROUP EFFECT IN INFLATION MODEL";
PROC GENMOD DATA=work.MA; 
  MODEL Nerror = agegroup over75 / LINK=LOG DIST=ZINB;
  ZEROMODEL agegroup / LINK=LOGIT; 
* Requesting log intercept to be transformed into count;
  ESTIMATE "Count Intercept for YA"       intercept 1 agegroup 0 over75  0;
  ESTIMATE "Count Intercept for OA Age70" intercept 1 agegroup 1 over75 -5;
  ESTIMATE "Count Intercept for OA Age75" intercept 1 agegroup 1 over75  0;
  ESTIMATE "Count Intercept for OA Age80" intercept 1 agegroup 1 over75  5;
  CONTRAST "DF=2 Total Age Effect"  agegroup 1, over75 1;
RUN; TITLE1; 

* Close output;
PROC PRINTTO; RUN; ODS HTML CLOSE; ODS HTML;


***** HURDLE POISSON AND NEGATIVE BINOMIAL UNIVARIATE MODELS FOR ERRORS;

* Zero-truncated distribution of number of errors;
PROC UNIVARIATE DATA=work.MA; WHERE Nerror>0; VAR Nerror; HISTOGRAM Nerror;
RUN; 

* Save SAS syntax and output to permanent text file;
PROC PRINTTO PRINT="&example.\2_Hurdle_Univariate_Models.txt" 
			 LOG=  "&example.\2_Hurdle_Univariate_Models.txt"; 
RUN;

TITLE1 "EMPTY MODEL PREDICTING ERRORS DV, LOG LINK, HURDLE POISSON DIST";
PROC NLMIXED DATA=work.MA; 
* List all parms to estimate and give start values;
  PARMS B0c=0 B0h=0;
  yHurdle = EXP(B0h)/(1+EXP(B0h)); * Predict 0;
  yCount = EXP(B0c);			   * Predict something if not 0;
* Build log-likelihood;
  IF Nerror=0 THEN LL = LOG(yHurdle);
  ELSE LL = LOG((1-yHurdle))+Nerror*LOG(yCount)-LGAMMA(Nerror+1)-yCount-LOG(1-EXP(-yCount));
  MODEL Nerror ~ GENERAL(LL);
* Requesting log intercept to be transformed into count;
  ESTIMATE "Probability: Intercept for Hurdle" yHurdle;
  ESTIMATE "Count: Intercept for Something"    yCount;
RUN; TITLE1; 

TITLE1 "EMPTY MODEL PREDICTING ERRORS DV, LOG LINK, HURDLE NEGATIVE BINOMIAL DIST";
PROC NLMIXED DATA=work.MA; 
* List all parms to estimate and give start values;
  PARMS B0c=0 B0h=0 scale=1;
  yHurdle = EXP(B0h)/(1+EXP(B0h)); * Predict 0;
  yCount = EXP(B0c);			   * Predict something if not 0;
* Build scale factor;
  NN=1/scale;
  Log_Choose=LGAMMA(Nerror+NN)-LGAMMA(Nerror+1)-LGAMMA(NN);
* Build log-likelihood;
  IF Nerror=0 THEN LL = LOG(yHurdle);
  ELSE LL = LOG((1-yHurdle))+ Nerror*LOG(yCount*scale/(1+yCount*scale))-LOG(1+yCount*scale)/scale+Log_Choose
                            -(LOG(1-(1/(1+yCount*scale))**(1/scale)));
  MODEL Nerror ~ GENERAL(LL);
* Requesting log intercept to be transformed into count;
  ESTIMATE "Probability: Intercept for Hurdle" yHurdle;
  ESTIMATE "Count: Intercept for Something"    yCount;
RUN; TITLE1; 

TITLE1 "AGE PIECEWISE LINEAR MODEL PREDICTING ERRORS DV, LOG LINK, HURDLE NEGATIVE BINOMIAL DIST";
PROC NLMIXED DATA=work.MA; 
* List all parms to estimate and give start values;
  PARMS B0c=0 B0h=0 scale=1 Bagec=0 Boverc=0 Bageh=0;
  yHurdle = EXP(B0h + Bageh*agegroup)/(1+EXP(B0h + Bageh*agegroup)); * Predict 0/something;
  yCount = EXP(B0c + Bagec*agegroup + Boverc*over75);	* Predict something if not 0;
* Build scale factor;
  NN=1/scale;
  Log_Choose=LGAMMA(Nerror+NN)-LGAMMA(Nerror+1)-LGAMMA(NN);
* Build log-likelihood;
  IF Nerror=0 THEN LL = LOG(yHurdle);
  ELSE LL = LOG((1-yHurdle))+ Nerror*LOG(yCount*scale/(1+yCount*scale))-LOG(1+yCount*scale)/scale+Log_Choose
                            -(LOG(1-(1/(1+yCount*scale))**(1/scale)));
  MODEL Nerror ~ GENERAL(LL);
RUN; TITLE1; 

* Close output;
PROC PRINTTO; RUN; ODS HTML CLOSE; ODS HTML;


***** TWO-PART NORMAL AND LOG-NORMAL UNIVARIATE MODELS FOR ERRORS;

* Zero-truncated distribution of number of errors;
PROC UNIVARIATE DATA=work.MA; WHERE Nerror>0; VAR Nerror; 
	HISTOGRAM Nerror / NORMAL(MU=EST SIGMA=EST) LOGNORMAL(SHAPE=EST SCALE=EST) GAMMA(SHAPE=EST SCALE=EST);
RUN; 

* Recoding and stacking two-part outcome;
DATA work.MAstack; SET work.MA;
	    IF Nerror=0 THEN DO; dv0=0; dvA=.; END;
   ELSE IF Nerror>0 THEN DO; dv0=1; dvA=Nerror; END;
	DistN=4; DistL=4;  DistG=4; LinkN=2; LinkL=2; LinkG=2; DV=dv0; Ind0=1; IndA=0; OUTPUT; * Binary;
    DistN=1; DistL=11; DistG=5; LinkN=1; LinkL=1; LinkG=4; DV=dvA; Ind0=0; IndA=1; OUTPUT; * Normal,Log-Normal,Gamma;
RUN;

* Save SAS syntax and output to permanent text file;
PROC PRINTTO PRINT="&example.\2_Two-Part_Univariate_Models.txt" 
			 LOG=  "&example.\2_Two-Part_Univariate_Models.txt"; 
RUN;

TITLE1 "EMPTY MODEL PREDICTING TWO-PART DV, NORMAL AMOUNT DIST";
PROC GLIMMIX DATA=work.MAstack NOCLPRINT GRADIENT METHOD=QUAD; 
  MODEL DV = Ind0 IndA / NOINT SOLUTION LINK=BYOBS(LinkN) DIST=BYOBS(DistN);
RUN; TITLE1; 

TITLE1 "EMPTY MODEL PREDICTING TWO-PART DV, LOG-NORMAL AMOUNT DIST";
PROC GLIMMIX DATA=work.MAstack NOCLPRINT GRADIENT METHOD=QUAD; 
  MODEL DV = Ind0 IndA / NOINT SOLUTION LINK=BYOBS(LinkL) DIST=BYOBS(DistL);
RUN; TITLE1; 

TITLE1 "EMPTY MODEL PREDICTING TWO-PART DV, GAMMA AMOUNT DIST";
PROC GLIMMIX DATA=work.MAstack NOCLPRINT GRADIENT METHOD=QUAD; 
  MODEL DV = Ind0 IndA / NOINT SOLUTION LINK=BYOBS(LinkG) DIST=BYOBS(DistG);
RUN; TITLE1; 

TITLE1 "AGE PIECEWISE LINEAR MODEL PREDICTING TWO-PART DV, NORMAL AMOUNT DIST";
PROC GLIMMIX DATA=work.MAstack NOCLPRINT GRADIENT METHOD=QUAD; 
  MODEL DV = Ind0 IndA Ind0*agegroup IndA*agegroup IndA*over75 / NOINT SOLUTION LINK=BYOBS(LinkN) DIST=BYOBS(DistN);
RUN; TITLE1; 

TITLE1 "AGE PIECEWISE LINEAR MODEL PREDICTING TWO-PART DV, LOG-NORMAL AMOUNT DIST";
PROC GLIMMIX DATA=work.MAstack NOCLPRINT GRADIENT METHOD=QUAD; 
  MODEL DV = Ind0 IndA Ind0*agegroup IndA*agegroup IndA*over75 / NOINT SOLUTION LINK=BYOBS(LinkL) DIST=BYOBS(DistL);
RUN; TITLE1; 

TITLE1 "AGE PIECEWISE LINEAR MODEL PREDICTING TWO-PART DV, GAMMA AMOUNT DIST";
PROC GLIMMIX DATA=work.MAstack NOCLPRINT GRADIENT METHOD=QUAD; 
  MODEL DV = Ind0 IndA Ind0*agegroup IndA*agegroup IndA*over75 / NOINT SOLUTION LINK=BYOBS(LinkG) DIST=BYOBS(DistG);
RUN; TITLE1; 

* Close output;
PROC PRINTTO; RUN; ODS HTML CLOSE; ODS HTML;


